/*
*      SCCS:   @(#)common/vprog/wmake.c	Rel 1.1 (2/5/92)
*
*	UniSoft Ltd., London, England
*
* (C) Copyright 1991 X/Open Company Limited
*
* All rights reserved.  No part of this source code may be reproduced,
* stored in a retrieval system, or transmitted, in any form or by any
* means, electronic, mechanical, photocopying, recording or otherwise,
* except as stated in the end-user licence agreement, without the prior
* permission of the copyright owners.
*
* X/Open and the 'X' symbol are trademarks of X/Open Company Limited in
* the UK and other countries.
*/

char	sccsid[] = "@(#)common/wrpt/wmake.c Rel 1.1 (2/5/92)";
char	*copyright[] = {
		"(C) Copyright 1992 X/Open Company Limited",
		"All Rights Reserved."
};

/***********************************************************************

NAME:		wmake.c

PROJECT:	VSW (X/OPEN Window Management Test Suite)
MODULE CONTAINS:
SYNOPSIS	Executes pmake if tet_pname
		is "wbuild" and pclean if
		tet_pname is "wclean".
			
DESCRIPTION:
RELATED DOCUMENTS:
AUTHOR:			Calum Laing, Unisoft Ltd.
DATE CREATED:		2nd November, 1990.
MODIFICATIONS:
		Geoff Clare, 8 Jan 1992
			Check for null return from tet_getvar().

		Stuart Boutell, 9 Jan 1992
			Transported from VSX for use in VSW.
				-remove unnecessary environment processing.

************************************************************************/
#include "../wrpt/std.h"
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/types.h>
#include	<sys/wait.h>
#include	<tet_api.h>

#define 	CMDBUFSZ	256
#define	BUILD_NAME	"wbuild.exec"
#define	CLEAN_NAME	"wclean.exec"

private	char	cmdbuf[CMDBUFSZ];	/* Command used to execute vmake.sh 		*/

private	char	*resfile="wmake_xres"; /* Name of file containing tool output	*/

private	void	startfunc();
private	void	toolfunc();
private void	tool_config();

public	void	(*tet_startup)() = startfunc;
public	void	(*tet_cleanup)() = NULL;

public	struct tet_testlist tet_testlist[] = {
	toolfunc, 0,
	NULL, 0
};

private void
startfunc()
{
	char *args;

	if(( args = getenv("VSW_ARGS")) == NULL ) {
		args = "";
	}

	if(strcmp(tet_pname, BUILD_NAME) == 0) {
		tool_config(args, "/xtest/bin/pmake ");
	}
	else if(strcmp(tet_pname, CLEAN_NAME ) == 0) {
		tool_config(args, "/xtest/bin/pclean ");
	} 
	else  {
		tet_delete(1,"Tool file name not set correctly.");
		return;
	}
}

private void
toolfunc()
{
	int		status = 0;
	FILE	*log;
	char	fbuf[60];

	(void) fflush(stdout);
	(void) fflush(stderr);

	status = system(cmdbuf);

	if (WIFEXITED(status) && WEXITSTATUS(status) == 0)
		tet_result(TET_PASS);
	else
	{
		if( (log = fopen( resfile, "r")) == NULL) {
			tet_infoline("fopen() failed to open build output file.");
		} else {
			while( fgets( fbuf, sizeof fbuf, log) != NULL)
				tet_infoline(fbuf);
			(void) fclose(log);
		}
		tet_result(TET_FAIL);
	}
	(void) unlink(resfile);
}

private void
tool_config(target, tool)
	char	*target, *tool;
{
	char	*s;
	char	*cvar;
	char	*ep;
	char	**ev;

	int		space;
	
/* Construct the shell command */

	if(( s = getenv("TET_ROOT")) == NULL || *s == '\0') {
		tet_delete(1,"no TET_ROOT in environment.");
		return;
	}
	
	(void) strcpy(cmdbuf, s);
	space = CMDBUFSZ - (int) strlen(s);

	(void) strncat(cmdbuf, tool,	(size_t)space); space -= (int)strlen(tool);
	(void) strncat(cmdbuf, target,	(size_t)space); space -= (int) strlen(target);
	(void) strncat(cmdbuf, s=" > ",	(size_t)space); space -= (int) strlen(s);
	(void) strncat(cmdbuf, resfile,	(size_t)space); space -= (int) strlen(resfile);
	(void) strncat(cmdbuf, s=" 2>&1", (size_t)space); space -= (int) strlen(s);
	
	if (space <= 0) {
		tet_delete(1,"make command too long for internal buffer");
		return;
	}
}
